@@ -1,4 +1,4 @@
-/* $Id: global.c,v 1.26 2003/03/09 22:04:29 mikpe Exp $
+/* $Id: global.c,v 1.29 2003/05/21 18:31:58 mikpe Exp $
  * global.c
  *
  * usage: ./global [sampling_interval_usec [sleep_interval_sec]]
@@ -173,6 +173,11 @@ static void setup_control(struct perfctr_cpu_control *control)
       case PERFCTR_X86_GENERIC:
 	nractrs = 0;		/* no PMCs available */
 	break;
+      case PERFCTR_X86_AMD_K8:
+	/* RETIRED_FPU_INSTRS, Unit Mask "x87 instrs", any CPL, Enable */
+	evntsel0 = 0xCB | (0x01 << 8) | (3 << 16) | (1 << 22);
+	break;
+#if !defined(__x86_64__)
       case PERFCTR_X86_INTEL_P5:
       case PERFCTR_X86_INTEL_P5MMX:
       case PERFCTR_X86_CYRIX_MII:
@@ -182,6 +187,7 @@ static void setup_control(struct perfctr_cpu_control *control)
       case PERFCTR_X86_INTEL_P6:
       case PERFCTR_X86_INTEL_PII:
       case PERFCTR_X86_INTEL_PIII:
+      case PERFCTR_X86_INTEL_PENTM:
 	/* note: FLOPS is only available in PERFCTR0 */
 	/* event 0xC1 (FLOPS), any CPL, Enable */
 	evntsel0 = 0xC1 | (3 << 16) | (1 << 22);
@@ -192,10 +198,6 @@ static void setup_control(struct perfctr_cpu_control *control)
 	/* event 0xC0 (RETIRED_INSTRUCTIONS), any CPL, Enable */
 	evntsel0 = 0xC0 | (3 << 16) | (1 << 22);
 	break;
-      case PERFCTR_X86_AMD_K8:
-	/* RETIRED_FPU_INSTRS, Unit Mask "x87 instrs", any CPL, Enable */
-	evntsel0 = 0xCB | (0x01 << 8) | (3 << 16) | (1 << 22);
-	break;
       case PERFCTR_X86_WINCHIP_C6:
 	counting_mips = 1;	/* can't count FLOPS */
 	tsc_on = 0;		/* no working TSC available */
@@ -223,6 +225,7 @@ static void setup_control(struct perfctr_cpu_control *control)
 	evntsel0 = (0x3 << 16) | (5 << 13) | (1 << 12);
 	control->p4.escr[0] = (0xC << 25) | (1 << 9) | (1 << 2);
 	break;
+#endif
       default:
 	fprintf(stderr, "cpu_type %u (%s) not supported\n",
 		info.cpu_type, perfctr_info_cpu_name(&info));
