@@ -1,4 +1,4 @@
-/* $Id: perfex.c,v 1.25 2003/03/10 22:07:58 mikpe Exp $
+/* $Id: perfex.c,v 1.27 2003/05/21 18:31:58 mikpe Exp $
  * perfex.c
  *
  * NAME
@@ -122,6 +122,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <getopt.h>
+#include <stddef.h>	/* for offsetof() */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>	/* for strerror() */
@@ -144,7 +145,12 @@
 struct cmsg_fd {
     struct cmsghdr hdr;
     int fd;
+    /* 64-bit machines pad here, which causes problems since
+       the kernel derives the number of fds from the size.
+       The CMSG_FD_TRUE_SIZE macro gives the true payload size. */
 };
+#define CMSG_FD_TRUE_SIZE	(offsetof(struct cmsg_fd, fd) + sizeof(int))
+#define CMSG_FD_PADDED_SIZE	sizeof(struct cmsg_fd)
 
 static int my_send(int sock, int fd, int status)
 {
@@ -167,12 +173,12 @@ static int my_send(int sock, int fd, int status)
 	msg.msg_control = 0;
 	msg.msg_controllen = 0;
     } else {
-	cmsg_fd.hdr.cmsg_len = sizeof cmsg_fd;
+	cmsg_fd.hdr.cmsg_len = CMSG_FD_TRUE_SIZE;
 	cmsg_fd.hdr.cmsg_level = SOL_SOCKET;
 	cmsg_fd.hdr.cmsg_type = SCM_RIGHTS;
 	cmsg_fd.fd = fd;
 	msg.msg_control = &cmsg_fd;
-	msg.msg_controllen = sizeof cmsg_fd;
+	msg.msg_controllen = CMSG_FD_TRUE_SIZE;
     }
     return sendmsg(sock, &msg, 0) == sizeof buf ? 0 : -1;
 }
@@ -206,17 +212,17 @@ static int my_receive(int sock, int *fd)
 
     memset(&cmsg_fd, ~0, sizeof cmsg_fd);
     msg.msg_control = &cmsg_fd;
-    msg.msg_controllen = sizeof cmsg_fd;
+    msg.msg_controllen = CMSG_FD_TRUE_SIZE;
 
     if( recvmsg(sock, &msg, 0) != sizeof buf )
 	return -1;
 
     if( buf[0] == 0 &&
 	msg.msg_control == &cmsg_fd &&
-	msg.msg_controllen == sizeof cmsg_fd &&
+	msg.msg_controllen == CMSG_FD_PADDED_SIZE &&
 	cmsg_fd.hdr.cmsg_type == SCM_RIGHTS &&
 	cmsg_fd.hdr.cmsg_level == SOL_SOCKET &&
-	cmsg_fd.hdr.cmsg_len == sizeof cmsg_fd &&
+	cmsg_fd.hdr.cmsg_len == CMSG_FD_TRUE_SIZE &&
 	cmsg_fd.fd >= 0 ) {
 	*fd = cmsg_fd.fd;
 	return 0;
@@ -300,17 +306,21 @@ static int do_parent(int sock, int child_pid, FILE *resfile)
     for(i = 0; i < nrctrs; ++i) {
 	fprintf(resfile, "event 0x%08X",
 		kstate->cpu_state.control.evntsel[i]);
+#if !defined(__x86_64__)
 	if( kstate->cpu_state.control.p4.escr[i] )
 	    fprintf(resfile, "/0x%08X",
 		    kstate->cpu_state.control.p4.escr[i]);
+#endif
 	fprintf(resfile, "\t%19lld\n", kstate->cpu_state.sum.pmc[i]);
     }
+#if !defined(__x86_64__)
     if( kstate->cpu_state.control.p4.pebs_enable )
 	fprintf(resfile, "PEBS_ENABLE 0x%08X\n",
 		kstate->cpu_state.control.p4.pebs_enable);
     if( kstate->cpu_state.control.p4.pebs_matrix_vert )
 	fprintf(resfile, "PEBS_MATRIX_VERT 0x%08X\n",
 		kstate->cpu_state.control.p4.pebs_matrix_vert);
+#endif
 
     munmap((void*)kstate, PAGE_SIZE);
     return WEXITSTATUS(child_status);
@@ -480,6 +490,7 @@ static int parse_event_spec(const char *arg, unsigned int *evntsel,
     return endp[0] != '\0';
 }
 
+#if !defined(__x86_64__)
 static int parse_value(const char *arg, unsigned int *value)
 {
     char *endp;
@@ -487,6 +498,7 @@ static int parse_value(const char *arg, unsigned int *value)
     *value = strtoul(arg, &endp, 16);
     return endp[0] != '\0';
 }
+#endif
 
 int main(int argc, char **argv)
 {
@@ -494,7 +506,9 @@ int main(int argc, char **argv)
     struct vperfctr_control control;
     int n;
     unsigned int spec_evntsel, spec_escr, spec_pmc;
+#if !defined(__x86_64__)
     unsigned int spec_value;
+#endif
     FILE *resfile;
 
     /* prime info, as we'll need it in most cases */
@@ -546,10 +560,13 @@ int main(int argc, char **argv)
 	    if( spec_pmc == (unsigned int)-1 )
 		spec_pmc = n;
 	    control.cpu_control.evntsel[n] = spec_evntsel;
+#if !defined(__x86_64__)
 	    control.cpu_control.p4.escr[n] = spec_escr;
+#endif
 	    control.cpu_control.pmc_map[n] = spec_pmc;
 	    control.cpu_control.nractrs = ++n;
 	    continue;
+#if !defined(__x86_64__)
 	  case 1:
 	    if( parse_value(optarg, &spec_value) ) {
 		fprintf(stderr, "perfex: invalid value: '%s'\n", optarg);
@@ -564,6 +581,7 @@ int main(int argc, char **argv)
 	    }
 	    control.cpu_control.p4.pebs_matrix_vert = spec_value;
 	    continue;
+#endif
 	  default:
 	    do_usage();
 	    return 1;
