@@ -1,4 +1,4 @@
-/* $Id: self.c,v 1.26 2003/03/09 22:04:08 mikpe Exp $
+/* $Id: self.c,v 1.29 2003/05/21 18:31:58 mikpe Exp $
  * self.c
  *
  * This test program illustrates how a process may use the
@@ -73,6 +73,7 @@ void do_setup(void)
       case PERFCTR_X86_GENERIC:
 	nractrs = 0;		/* no PMCs available */
 	break;
+#if !defined(__x86_64__)
       case PERFCTR_X86_INTEL_P5:
       case PERFCTR_X86_INTEL_P5MMX:
       case PERFCTR_X86_CYRIX_MII:
@@ -82,11 +83,14 @@ void do_setup(void)
       case PERFCTR_X86_INTEL_P6:
       case PERFCTR_X86_INTEL_PII:
       case PERFCTR_X86_INTEL_PIII:
+      case PERFCTR_X86_INTEL_PENTM:
       case PERFCTR_X86_AMD_K7:
+#endif
       case PERFCTR_X86_AMD_K8:
 	/* event 0xC0 (INST_RETIRED), count at CPL > 0, Enable */
 	evntsel0 = 0xC0 | (1 << 16) | (1 << 22);
 	break;
+#if !defined(__x86_64__)
       case PERFCTR_X86_WINCHIP_C6:
 	tsc_on = 0;		/* no working TSC available */
 	evntsel0 = 0x02;	/* X86_INSTRUCTIONS */
@@ -108,6 +112,7 @@ void do_setup(void)
 	/* CRU_ESCR0: event 2 (instr_retired), NBOGUSNTAG, CPL>0 */
 	control.cpu_control.p4.escr[0] = (2 << 25) | (1 << 9) | (1 << 2);
 	break;
+#endif
       default:
 	fprintf(stderr, "cpu type %u (%s) not supported\n",
 		info.cpu_type, perfctr_info_cpu_name(&info));
