@@ -1,4 +1,4 @@
-/* $Id: signal.c,v 1.14 2003/03/09 22:03:19 mikpe Exp $
+/* $Id: signal.c,v 1.16 2003/05/16 12:30:15 mikpe Exp $
  * signal.c
  *
  * This test program illustrates how performance counter overflow
@@ -14,7 +14,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#if !defined(__x86_64__)
 #include <asm/sigcontext.h>
+#endif
 #include <asm/ucontext.h>	/* _not_ the broken <sys/ucontext.h> */
 #include "libperfctr.h"
 
@@ -57,7 +59,11 @@ static void on_sigio(int sig, siginfo_t *si, void *puc)
     }
     uc = puc;
     mc = &uc->uc_mcontext;
+#ifdef __x86_64__
+    pc = mc->rip;
+#else
     pc = mc->eip;	/* clearly more readable than glibc's mc->gregs[14] */
+#endif
     if( !vperfctr_is_running(vperfctr) ) {
 	/*
 	 * My theory is that this happens if a perfctr overflowed
@@ -102,9 +108,11 @@ static void do_control(void)
     memset(&control, 0, sizeof control);
 
     switch( info.cpu_type ) {
+#if !defined(__x86_64__)
       case PERFCTR_X86_INTEL_P6:
       case PERFCTR_X86_INTEL_PII:
       case PERFCTR_X86_INTEL_PIII:
+      case PERFCTR_X86_INTEL_PENTM:
 	/* FLOPS, USR, ENable, INT */
 	evntsel0 = 0xC1 | (1 << 16) | (1 << 22) | (1 << 20);
 	/* BR_TAKEN_RETIRED, USR, INT */
@@ -116,12 +124,6 @@ static void do_control(void)
 	/* RETIRED_TAKEN_BRANCHES, USR, INT */
 	evntsel1 = 0xC4 | (1 << 16) | (1 << 22) | (1 << 20);
 	break;
-      case PERFCTR_X86_AMD_K8:
-	/* RETIRED_FPU_INSTRS, Unit Mask "x87 instrs", any CPL, Enable */
-	evntsel0 = 0xCB | (0x01 << 8) | (3 << 16) | (1 << 22);
-	/* RETIRED_TAKEN_BRANCHES, USR, INT */
-	evntsel1 = 0xC4 | (1 << 16) | (1 << 22) | (1 << 20);
-	break;
       case PERFCTR_X86_INTEL_P4:
       case PERFCTR_X86_INTEL_P4M2:
 	nractrs = 1;
@@ -138,6 +140,13 @@ static void do_control(void)
 	evntsel1 = (1 << 26) | (0x3 << 16) | (5 << 13) | (1 << 12);
 	control.cpu_control.p4.escr[2] = (6 << 25) | (((1 << 3)|(1 << 2)) << 9) | (1 << 2);
 	break;
+#endif
+      case PERFCTR_X86_AMD_K8:
+	/* RETIRED_FPU_INSTRS, Unit Mask "x87 instrs", any CPL, Enable, INT */
+	evntsel0 = 0xCB | (0x01 << 8) | (3 << 16) | (1 << 22) | (1 << 20);
+	/* RETIRED_TAKEN_BRANCHES, USR, Enable, INT */
+	evntsel1 = 0xC4 | (1 << 16) | (1 << 22) | (1 << 20);
+	break;
       default:
 	printf("%s: unsupported cpu type %u\n", __FUNCTION__, info.cpu_type);
 	exit(1);
