@@ -1,4 +1,4 @@
-/* $Id: x86.c,v 1.91 2003/03/09 16:44:04 mikpe Exp $
+/* $Id: x86.c,v 1.100 2003/06/01 12:27:20 mikpe Exp $
  * x86 performance-monitoring counters driver.
  *
  * Copyright (C) 1999-2003  Mikael Pettersson
@@ -13,6 +13,7 @@
 
 #include <asm/msr.h>
 #undef MSR_P6_PERFCTR0
+#undef MSR_IA32_MISC_ENABLE
 
 #include "compat.h"
 #include "x86_compat.h"
@@ -55,10 +56,12 @@ static struct per_cpu_cache per_cpu_cache[NR_CPUS] __cacheline_aligned;
 #define MSR_K7_EVNTSEL0		0xC0010000	/* .. 0xC0010003 */
 #define MSR_K7_PERFCTR0		0xC0010004	/* .. 0xC0010007 */
 
+/* Intel P4, Intel Pentium M */
+#define MSR_IA32_MISC_ENABLE	0x1A0
+#define MSR_IA32_MISC_ENABLE_PERF_AVAIL (1<<7)	/* read-only status bit */
+#define MSR_IA32_MISC_ENABLE_PEBS_UNAVAIL (1<<12) /* read-only status bit */
+
 /* Intel P4 */
-#define MSR_P4_MISC_ENABLE	0x1A0
-#define MSR_P4_MISC_ENABLE_PERF_AVAIL (1<<7)	/* read-only status bit */
-#define MSR_P4_MISC_ENABLE_PEBS_UNAVAIL (1<<12) /* read-only status bit */
 #define MSR_P4_PERFCTR0		0x300		/* .. 0x311 */
 #define MSR_P4_CCCR0		0x360		/* .. 0x371 */
 #define MSR_P4_ESCR0		0x3A0		/* .. 0x3E1, with some gaps */
@@ -76,6 +79,7 @@ static struct per_cpu_cache per_cpu_cache[NR_CPUS] __cacheline_aligned;
 #define P4_CCCR_CASCADE		0x40000000
 #define P4_CCCR_OVF_PMI_T1	0x08000000
 #define P4_CCCR_OVF_PMI_T0	0x04000000
+#define P4_CCCR_FORCE_OVF	0x02000000
 #define P4_CCCR_ACTIVE_THREAD	0x00030000
 #define P4_CCCR_ENABLE		0x00001000
 #define P4_CCCR_ESCR_SELECT(X)	(((X) >> 13) & 0x7)
@@ -243,7 +247,7 @@ static void p5_read_counters(const struct perfctr_cpu_state *state,
 	}
 }
 
-/* used by P5MMX, MII, C6, VC3, P6, K7, P4 */
+/* used by all except pre-MMX P5 */
 static void rdpmc_read_counters(const struct perfctr_cpu_state *state,
 				struct perfctr_low_ctrs *ctrs)
 {
@@ -512,7 +516,7 @@ static void p6_like_write_control(const struct perfctr_cpu_state *state,
 	cpu->k1.id = state->k1.id;
 }
 
-/* shared with VC3 */
+/* shared with VC3, Generic*/
 static void p6_write_control(const struct perfctr_cpu_state *state)
 {
 	p6_like_write_control(state, MSR_P6_EVNTSEL0);
@@ -535,12 +539,8 @@ static void p6_clear_counters(void)
  * - The EVNTSEL MSRs are symmetric since each has its own enable bit.
  * - Publicly available documentation is incomplete.
  *
- * The K8 does not yet have detailed public documentation, but the
- * MSR layouts and assignments are known to be identical to the K7.
- * Official K7 events appear to be supported. Unofficial K7 events
- * (those that have been removed from later revisions of #22007)
- * appear to be mostly supported. The K8 also adds some events
- * that are known to not exist in the K7.
+ * The K8 has the same hardware layout as the K7. It also has
+ * better documentation and a different set of available events.
  */
 
 static int k7_check_control(struct perfctr_cpu_state *state)
@@ -608,6 +608,21 @@ static void vc3_clear_counters(void)
  * Intel Pentium 4.
  * Current implementation restrictions:
  * - No DS/PEBS support.
+ *
+ * Known quirks:
+ * - OVF_PMI+FORCE_OVF counters must have an ireset value of -1.
+ *   This allows the regular overflow check to also handle FORCE_OVF
+ *   counters. Not having this restriction would lead to MAJOR
+ *   complications in the driver's "detect overflow counters" code.
+ *   There is no loss of functionality since the ireset value doesn't
+ *   affect the counter's PMI rate for FORCE_OVF counters.
+ * - In experiments with FORCE_OVF counters, and regular OVF_PMI
+ *   counters with small ireset values between -8 and -1, it appears
+ *   that the faulting instruction is subjected to a new PMI before
+ *   it can complete, ad infinitum. This occurs even though the driver
+ *   clears the CCCR (and in testing also the ESCR) and invokes a
+ *   user-space signal handler before restoring the CCCR and resuming
+ *   the instruction.
  */
 
 /*
@@ -710,6 +725,9 @@ static int p4_check_control(struct perfctr_cpu_state *state)
 		if( cccr_val & P4_CCCR_OVF_PMI_T0 ) {
 			if( i < nractrs )
 				return -EINVAL;
+			if( (cccr_val & P4_CCCR_FORCE_OVF) &&
+			    state->control.ireset[i] != -1 )
+				return -EINVAL;
 		} else {
 			if( i >= nractrs )
 				return -EINVAL;
@@ -772,8 +790,6 @@ static void p4_isuspend(struct perfctr_cpu_state *state)
 	}
 	/* cpu->k1.id is still == state->k1.id */
 	set_isuspend_cpu(state, cpu);
-	/* another P4 quirk: must unmask LVTPC */
-	apic_write(APIC_LVTPC, LOCAL_PERFCTR_VECTOR);
 }
 
 /* PRE: perfctr_cstatus_has_ictrs(state->cstatus) != 0 */
@@ -875,16 +891,6 @@ static int generic_check_control(struct perfctr_cpu_state *state)
 	return 0;
 }
 
-static void generic_write_control(const struct perfctr_cpu_state *state)
-{
-}
-
-static void generic_read_counters(const struct perfctr_cpu_state *state,
-				  struct perfctr_low_ctrs *ctrs)
-{
-	rdtscl(ctrs->tsc);
-}
-
 static void generic_clear_counters(void)
 {
 }
@@ -980,6 +986,7 @@ void perfctr_cpu_ireload(struct perfctr_cpu_state *state)
 }
 
 /* PRE: the counters have been suspended and sampled by perfctr_cpu_suspend() */
+static int is_p4;
 unsigned int perfctr_cpu_identify_overflow(struct perfctr_cpu_state *state)
 {
 	unsigned int cstatus, nrctrs, pmc, pmc_mask;
@@ -999,6 +1006,8 @@ unsigned int perfctr_cpu_identify_overflow(struct perfctr_cpu_state *state)
 			   in order to stop the i-mode counters. */
 		}
 	}
+	if( is_p4 )
+		apic_write(APIC_LVTPC, LOCAL_PERFCTR_VECTOR);
 	return pmc_mask;
 }
 #endif	/* PERFCTR_INTERRUPT_SUPPORT */
@@ -1164,40 +1173,41 @@ static void __init finalise_backpatching(void)
 
 #ifdef CONFIG_SMP
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
-static inline int __init p4_ht_mask_setup(void)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,15)
+static inline int __init p4_ht_finalise(unsigned long forbidden)
 {
 	printk(KERN_WARNING "perfctr/x86.c: hyper-threaded P4s detected:"
-	       " kernel too old! -- bailing out\n");
+	       " you need a newer kernel! -- bailing out\n");
 	return -ENODEV;
 }
 #else
 
 unsigned long perfctr_cpus_forbidden_mask;
 
-static void __init p4_ht_mask_setup_cpu(void *ignore)
+static inline int __init p4_ht_finalise(unsigned long forbidden)
+{
+	perfctr_cpus_forbidden_mask = forbidden;
+	printk(KERN_INFO "perfctr/x86.c: hyper-threaded P4s detected:"
+	       " restricting access for CPUs 0x%lX\n", forbidden);
+	return 0;
+}
+
+#endif
+
+static void __init p4_ht_mask_setup_cpu(void *forbiddenp)
 {
 	unsigned int local_apic_physical_id = cpuid_ebx(1) >> 24;
 	unsigned int logical_processor_id = local_apic_physical_id & 1;
 	if( logical_processor_id != 0 ) {
 		unsigned long bit = 1 << smp_processor_id();
-		atomic_set_mask(bit, &perfctr_cpus_forbidden_mask);
+		atomic_set_mask(bit, (unsigned long*)forbiddenp);
 	}
 }
 
-static inline int __init p4_ht_mask_setup(void)
-{
-	smp_call_function(p4_ht_mask_setup_cpu, NULL, 1, 1);
-	p4_ht_mask_setup_cpu(NULL);
-	printk(KERN_INFO "perfctr/x86.c: hyper-threaded P4s detected: restricting"
-	       " access for CPUs 0x%lX\n", perfctr_cpus_forbidden_mask);
-	return 0;
-}
-#endif
-
 static int __init p4_ht_init(void)
 {
 	unsigned int nr_siblings;
+	unsigned long forbidden;
 
 	if( !cpu_has_ht )
 		return 0;
@@ -1208,7 +1218,14 @@ static int __init p4_ht_init(void)
 		       nr_siblings);
 		return -ENODEV;
 	}
-	return (nr_siblings < 2) ? 0 : p4_ht_mask_setup();
+	if( nr_siblings < 2 )
+		return 0;
+	forbidden = 0;
+	smp_call_function(p4_ht_mask_setup_cpu, &forbidden, 1, 1);
+	p4_ht_mask_setup_cpu(&forbidden);
+	if( !forbidden )
+		return 0;
+	return p4_ht_finalise(forbidden);
 }
 #else /* SMP */
 #define p4_ht_init()	(0)
@@ -1216,6 +1233,14 @@ static int __init p4_ht_init(void)
 
 static int __init intel_init(void)
 {
+	static char p5_name[] __initdata = "Intel Pentium";
+	static char p5mmx_name[] __initdata = "Intel Pentium MMX";
+	static char ppro_name[] __initdata = "Intel Pentium Pro";
+	static char pii_name[] __initdata = "Intel Pentium II";
+	static char piii_name[] __initdata = "Intel Pentium III";
+	static char p4_name[] __initdata = "Intel Pentium 4";
+	static char p4m2_name[] __initdata = "Intel Pentium 4 Model 2";
+	static char pentm_name[] __initdata = "Intel Pentium M";
 	unsigned int misc_enable;
 
 	if( !cpu_has_tsc )
@@ -1224,6 +1249,7 @@ static int __init intel_init(void)
 	case 5:
 		if( cpu_has_mmx ) {
 			perfctr_info.cpu_type = PERFCTR_X86_INTEL_P5MMX;
+			perfctr_cpu_name = p5mmx_name;
 			read_counters = rdpmc_read_counters;
 
 			/* Avoid Pentium Erratum 74. */
@@ -1234,6 +1260,7 @@ static int __init intel_init(void)
 				perfctr_info.cpu_features &= ~PERFCTR_FEATURE_RDPMC;
 		} else {
 			perfctr_info.cpu_type = PERFCTR_X86_INTEL_P5;
+			perfctr_cpu_name = p5_name;
 			perfctr_info.cpu_features &= ~PERFCTR_FEATURE_RDPMC;
 			read_counters = p5_read_counters;
 		}
@@ -1243,12 +1270,24 @@ static int __init intel_init(void)
 		perfctr_p5_init_tests();
 		return 0;
 	case 6:
-		if( current_cpu_data.x86_model >= 7 )	/* PIII */
+		if( current_cpu_data.x86_model == 9 ) {	/* Pentium M */
+			/* Pentium M added the MISC_ENABLE MSR from P4. */
+			rdmsrl(MSR_IA32_MISC_ENABLE, misc_enable);
+			if( !(misc_enable & MSR_IA32_MISC_ENABLE_PERF_AVAIL) )
+				break;
+			/* Erratum Y3 probably does not apply since we
+			   read only the low 32 bits. */
+			perfctr_info.cpu_type = PERFCTR_X86_INTEL_PENTM;
+			perfctr_cpu_name = pentm_name;
+		} else if( current_cpu_data.x86_model >= 7 ) {	/* PIII */
 			perfctr_info.cpu_type = PERFCTR_X86_INTEL_PIII;
-		else if( current_cpu_data.x86_model >= 3 )	/* PII or Celeron */
+			perfctr_cpu_name = piii_name;
+		} else if( current_cpu_data.x86_model >= 3 ) {	/* PII or Celeron */
 			perfctr_info.cpu_type = PERFCTR_X86_INTEL_PII;
-		else {
+			perfctr_cpu_name = pii_name;
+		} else {
 			perfctr_info.cpu_type = PERFCTR_X86_INTEL_P6;
+			perfctr_cpu_name = ppro_name;
 
 			/* Avoid Pentium Pro Erratum 26. */
 			if( current_cpu_data.x86_mask < 9 )
@@ -1268,17 +1307,20 @@ static int __init intel_init(void)
 		perfctr_p6_init_tests();
 		return 0;
 	case 15:	/* Pentium 4 */
-		rdmsrl(MSR_P4_MISC_ENABLE, misc_enable);
-		if( !(misc_enable & MSR_P4_MISC_ENABLE_PERF_AVAIL) )
+		rdmsrl(MSR_IA32_MISC_ENABLE, misc_enable);
+		if( !(misc_enable & MSR_IA32_MISC_ENABLE_PERF_AVAIL) )
 			break;
 		if( p4_ht_init() != 0 )
 			break;
-		if( current_cpu_data.x86_model >= 2 )
+		if( current_cpu_data.x86_model >= 2 ) {
 			/* Model 2 changed the ESCR Event Mask programming
 			   details for several events. */
 			perfctr_info.cpu_type = PERFCTR_X86_INTEL_P4M2;
-		else
+			perfctr_cpu_name = p4m2_name;
+		} else {
 			perfctr_info.cpu_type = PERFCTR_X86_INTEL_P4;
+			perfctr_cpu_name = p4_name;
+		}
 		read_counters = rdpmc_read_counters;
 		write_control = p4_write_control;
 		check_control = p4_check_control;
@@ -1288,6 +1330,7 @@ static int __init intel_init(void)
 			perfctr_info.cpu_features |= PERFCTR_FEATURE_PCINT;
 			cpu_isuspend = p4_isuspend;
 			cpu_iresume = p4_iresume;
+			is_p4 = 1;
 		}
 #endif
 		perfctr_p4_init_tests();
@@ -1298,16 +1341,20 @@ static int __init intel_init(void)
 
 static int __init amd_init(void)
 {
+	static char k7_name[] __initdata = "AMD K7";
+	static char k8_name[] __initdata = "AMD K8";
 	if( !cpu_has_tsc )
 		return -ENODEV;
 	switch( current_cpu_data.x86 ) {
 	case 15: /* K8. Like a K7 with more events, roughly. */
 		perfctr_info.cpu_type = PERFCTR_X86_AMD_K8;
+		perfctr_cpu_name = k8_name;
 		break;
 	case 6:	/* K7. Model 1 does not have a local APIC.
 		   AMD Document #22007 Revision J hints that APIC-less
 		   K7s signal overflows as debug interrupts. */
 		perfctr_info.cpu_type = PERFCTR_X86_AMD_K7;
+		perfctr_cpu_name = k7_name;
 		break;
 	default:
 		return -ENODEV;
@@ -1329,11 +1376,13 @@ static int __init amd_init(void)
 
 static int __init cyrix_init(void)
 {
+	static char mii_name[] __initdata = "Cyrix 6x86MX/MII/III";
 	if( !cpu_has_tsc )
 		return -ENODEV;
 	switch( current_cpu_data.x86 ) {
 	case 6:	/* 6x86MX, MII, or III */
 		perfctr_info.cpu_type = PERFCTR_X86_CYRIX_MII;
+		perfctr_cpu_name = mii_name;
 		read_counters = rdpmc_read_counters;
 		write_control = p5_write_control;
 		check_control = mii_check_control;
@@ -1346,16 +1395,23 @@ static int __init cyrix_init(void)
 
 static int __init centaur_init(void)
 {
+#if !defined(CONFIG_X86_TSC)
+	static char wcc6_name[] __initdata = "WinChip C6";
+	static char wc2_name[] __initdata = "WinChip 2/3";
+#endif
+	static char vc3_name[] __initdata = "VIA C3";
 	switch( current_cpu_data.x86 ) {
 #if !defined(CONFIG_X86_TSC)
 	case 5:
 		switch( current_cpu_data.x86_model ) {
 		case 4: /* WinChip C6 */
 			perfctr_info.cpu_type = PERFCTR_X86_WINCHIP_C6;
+			perfctr_cpu_name = wcc6_name;
 			break;
 		case 8: /* WinChip 2, 2A, or 2B */
 		case 9: /* WinChip 3, a 2A with larger cache and lower voltage */
 			perfctr_info.cpu_type = PERFCTR_X86_WINCHIP_2;
+			perfctr_cpu_name = wc2_name;
 			break;
 		default:
 			return -ENODEV;
@@ -1381,10 +1437,13 @@ static int __init centaur_init(void)
 		case 7:	/* VIA C3 Samuel 2 or Ezra */
 		case 8:	/* VIA C3 Ezra-T */
 			break;
+		case 9: /* Nehemiah. No docs publicly available yet. */
+			/*FALLTHROUGH*/
 		default:
 			return -ENODEV;
 		}
 		perfctr_info.cpu_type = PERFCTR_X86_VIA_C3;
+		perfctr_cpu_name = vc3_name;
 		read_counters = rdpmc_read_counters;
 		write_control = p6_write_control;
 		check_control = vc3_check_control;
@@ -1397,50 +1456,20 @@ static int __init centaur_init(void)
 
 static int __init generic_init(void)
 {
+	static char generic_name[] __initdata = "Generic x86 with TSC";
 	if( !cpu_has_tsc )
 		return -ENODEV;
 	perfctr_info.cpu_features &= ~PERFCTR_FEATURE_RDPMC;
 	perfctr_info.cpu_type = PERFCTR_X86_GENERIC;
+	perfctr_cpu_name = generic_name;
 	check_control = generic_check_control;
-	write_control = generic_write_control;
-	read_counters = generic_read_counters;
+	write_control = p6_write_control;
+	read_counters = rdpmc_read_counters;
 	clear_counters = generic_clear_counters;
 	perfctr_generic_init_tests();
 	return 0;
 }
 
-static char generic_name[] __initdata = "Generic x86 with TSC";
-static char p5_name[] __initdata = "Intel Pentium";
-static char p5mmx_name[] __initdata = "Intel Pentium MMX";
-static char p6_name[] __initdata = "Intel Pentium Pro";
-static char pii_name[] __initdata = "Intel Pentium II";
-static char piii_name[] __initdata = "Intel Pentium III";
-static char mii_name[] __initdata = "Cyrix 6x86MX/MII/III";
-static char wcc6_name[] __initdata = "WinChip C6";
-static char wc2_name[] __initdata = "WinChip 2/3";
-static char k7_name[] __initdata = "AMD K7";
-static char vc3_name[] __initdata = "VIA C3";
-static char p4_name[] __initdata = "Intel Pentium 4";
-static char p4m2_name[] __initdata = "Intel Pentium 4 Model 2";
-static char k8_name[] __initdata = "AMD K8";
-
-char *perfctr_cpu_name[] __initdata = {
-	[PERFCTR_X86_GENERIC] generic_name,
-	[PERFCTR_X86_INTEL_P5] p5_name,
-	[PERFCTR_X86_INTEL_P5MMX] p5mmx_name,
-	[PERFCTR_X86_INTEL_P6] p6_name,
-	[PERFCTR_X86_INTEL_PII] pii_name,
-	[PERFCTR_X86_INTEL_PIII] piii_name,
-	[PERFCTR_X86_CYRIX_MII] mii_name,
-	[PERFCTR_X86_WINCHIP_C6] wcc6_name,
-	[PERFCTR_X86_WINCHIP_2] wc2_name,
-	[PERFCTR_X86_AMD_K7] k7_name,
-	[PERFCTR_X86_VIA_C3] vc3_name,
-	[PERFCTR_X86_INTEL_P4] p4_name,
-	[PERFCTR_X86_INTEL_P4M2] p4m2_name,
-	[PERFCTR_X86_AMD_K8] k8_name,
-};
-
 static void __init perfctr_cpu_init_one(void *ignore)
 {
 	/* PREEMPT note: when called via smp_call_function(),
@@ -1469,17 +1498,78 @@ static void __exit perfctr_cpu_exit_one(void *ignore)
 
 #if defined(NMI_LOCAL_APIC) && defined(CONFIG_PM)
 
-static void __init unregister_nmi_pmdev(void)
+static void perfctr_pm_suspend(void)
 {
-	if( nmi_pmdev ) {
-		apic_pm_unregister(nmi_pmdev);
-		nmi_pmdev = 0;
-	}
+	/* XXX: clear control registers */
+	printk("perfctr/x86: PM suspend\n");
 }
 
+static void perfctr_pm_resume(void)
+{
+	/* XXX: reload control registers */
+	printk("perfctr/x86: PM resume\n");
+}
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,67)
+
+#include <linux/device.h>
+
+static int perfctr_device_suspend(struct device *dev, u32 state, u32 level)
+{
+	if( level == SUSPEND_POWER_DOWN )
+		perfctr_pm_suspend();
+	return 0;
+}
+
+static int perfctr_device_resume(struct device *dev, u32 level)
+{
+	if( level == RESUME_POWER_ON )
+		perfctr_pm_resume();
+	return 0;
+}
+
+static struct device_driver perfctr_driver = {
+	.name		= "perfctr",
+	.bus		= &system_bus_type,
+	.resume		= perfctr_device_resume,
+	.suspend	= perfctr_device_suspend,
+};
+
+static struct sys_device device_perfctr = {
+	.name	= "perfctr",
+	.id	= 0,
+	.dev	= {
+		.name	= "perfctr",
+		.driver	= &perfctr_driver,
+	},
+};
+
+static void __init x86_pm_init(void)
+{
+	if( cpu_has_apic )
+		device_perfctr.dev.parent = &device_lapic.dev;
+	driver_register(&perfctr_driver);
+	sys_device_register(&device_perfctr);
+}
+
+static void __exit x86_pm_exit(void)
+{
+	sys_device_unregister(&device_perfctr);
+	driver_unregister(&perfctr_driver);
+}
+
+#else	/* 2.4 kernel */
+
 static int x86_pm_callback(struct pm_dev *dev, pm_request_t rqst, void *data)
 {
-	/* XXX: incomplete */
+	switch( rqst ) {
+	case PM_SUSPEND:
+		perfctr_pm_suspend();
+		break;
+	case PM_RESUME:
+		perfctr_pm_resume();
+		break;
+	}
 	return 0;
 }
 
@@ -1498,22 +1588,35 @@ static void __exit x86_pm_exit(void)
 	}
 }
 
+#endif	/* 2.4 kernel */
+
 #else
 
-static inline void unregister_nmi_pmdev(void) { }
 static inline void x86_pm_init(void) { }
 static inline void x86_pm_exit(void) { }
 
 #endif	/* NMI_LOCAL_APIC && CONFIG_PM */
 
-#if defined(NMI_LOCAL_APIC)
+#ifdef NMI_LOCAL_APIC
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,67)
+static void __init disable_lapic_nmi_watchdog(void)
+{
+#ifdef CONFIG_PM
+	if( nmi_pmdev ) {
+		apic_pm_unregister(nmi_pmdev);
+		nmi_pmdev = 0;
+	}
+#endif
+}
+#endif
 
 static void __init disable_nmi_watchdog(void)
 {
 	if( nmi_perfctr_msr ) {
 		nmi_perfctr_msr = 0;
-		printk(KERN_NOTICE "perfctr: disabled nmi_watchdog\n");
-		unregister_nmi_pmdev();
+		disable_lapic_nmi_watchdog();
+		printk(KERN_NOTICE "perfctr: disabled lapic_nmi_watchdog\n");
 	}
 }
 
