@@ -1,4 +1,4 @@
-/* $Id: x86_setup.c,v 1.36 2003/03/09 22:09:52 mikpe Exp $
+/* $Id: x86_setup.c,v 1.39 2003/04/13 12:43:48 mikpe Exp $
  * Performance-monitoring counters driver.
  * x86-specific kernel-resident code.
  *
@@ -16,7 +16,7 @@
 #include "compat.h"
 
 /* XXX: belongs to a virtual_compat.c file */
-#if defined(CONFIG_SMP) && LINUX_VERSION_CODE < KERNEL_VERSION(2,4,21) && LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0) && !defined(HAVE_SET_CPUS_ALLOWED)
+#if defined(CONFIG_SMP) && LINUX_VERSION_CODE < KERNEL_VERSION(2,4,21) && LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,15) && !defined(HAVE_SET_CPUS_ALLOWED)
 /**
  * set_cpus_allowed() - change a given task's processor affinity
  * @p: task to bind
@@ -56,10 +56,9 @@ EXPORT_SYMBOL(set_cpus_allowed);
 #endif
 
 #ifdef CONFIG_PERFCTR_VIRTUAL	/* XXX: actually generic, not x86-specific */
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,32)
-#include <linux/ptrace.h>
-#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,15)
-#include <linux/mm.h>
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,15)
+#include <linux/mm.h> /* for 2.4.15 and up, except 2.4.20-8-redhat */
+#include <linux/ptrace.h> /* for 2.5.32 and up, and 2.4.20-8-redhat */
 #else	/* < 2.4.15 */
 int ptrace_check_attach(struct task_struct *child, int kill)
 {
@@ -160,11 +159,11 @@ EXPORT_SYMBOL(perfctr_cpu_khz);
 #ifdef NMI_LOCAL_APIC
 EXPORT_SYMBOL(nmi_perfctr_msr);
 
-#ifdef CONFIG_PM
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,67) && defined(CONFIG_PM)
 EXPORT_SYMBOL(apic_pm_register);
 EXPORT_SYMBOL(apic_pm_unregister);
 EXPORT_SYMBOL(nmi_pmdev);
-#endif /* CONFIG_PM */
+#endif
 
 #endif /* NMI_LOCAL_APIC */
 
