@@ -1,4 +1,4 @@
-/* $Id: event_set_p4.c,v 1.1 2003/02/16 21:08:54 mikpe Exp $
+/* $Id: event_set_p4.c,v 1.3 2003/05/15 22:39:17 mikpe Exp $
  * Performance counter event descriptions for Intel P4.
  *
  * Copyright (C) 2003  Mikael Pettersson
@@ -35,23 +35,6 @@ enum escr_set {
     TC_ESCR_0_1,	/* CCCR 4/5/6/7 via ESCR select 0x01 */
 };
 
-static const struct perfctr_unit_mask_4 p4_um_branch_retired = {
-    { .type = perfctr_um_type_bitmask,
-      .default_value = 0x0C,	/* taken branches */
-      .nvalues = 4 },
-    { { 0x01, "MMNP:Branch Not-taken Predicted" },
-      { 0x02, "MMNM:Branch Not-taken Mispredicted" },
-      { 0x04, "MMTP:Branch Taken Predicted" },
-      { 0x08, "MMTM:Branch Taken Mispredicted" } }
-};
-
-static const struct perfctr_unit_mask_1 p4_um_mispred_branch_retired = {
-    { .type = perfctr_um_type_bitmask,
-      .default_value = 0x01,
-      .nvalues = 1 },
-    { { 0x01, "NBOGUS:The retired branch is not bogus" } }
-};
-
 static const struct perfctr_unit_mask_8 p4_um_TC_deliver_mode = {
     { .type = perfctr_um_type_bitmask,
       .default_value = 0x01,	/* DD */
@@ -201,17 +184,6 @@ static const struct perfctr_unit_mask_13 p4_um_BSQ = {
       { 0x2000, "MEM_TYPE2:Memory type encoding bit 2" } }
 };
 
-static const struct perfctr_unit_mask_5 p4_um_x87_assist = {
-    { .type = perfctr_um_type_bitmask,
-      .default_value = 0x1F,
-      .nvalues = 5 },
-    { { 0x01, "FPSU:FP stack underflow" },
-      { 0x02, "FPSO:FP stack overflow" },
-      { 0x04, "POAO:x87 output overflow" },
-      { 0x08, "POAU:x87 output underflow" },
-      { 0x10, "PREA:x87 input assist" } }
-};
-
 static const struct perfctr_unit_mask_1 p4_um_firm_uop = {
     { .type = perfctr_um_type_bitmask,
       .default_value = 0x8000,
@@ -227,15 +199,6 @@ static const struct perfctr_unit_mask_2 p4_um_x87_SIMD_moves_uop = {
       { 0x10, "ALLP2:count all x87/SIMD load uops" } }
 };
 
-static const struct perfctr_unit_mask_3 p4_um_machine_clear = {
-    { .type = perfctr_um_type_bitmask,
-      .default_value = 0x01,
-      .nvalues = 3 },
-    { { 0x01, "CLEAR:Count a portion of the cycles when the machine is cleared" },
-      { 0x04, "MOCLEAR:Count clears due to memory ordering issues" },
-      { 0x08, "SMCLEAR:Count clears due to self-modifying code issues" } }
-};
-
 static const struct perfctr_unit_mask_1 p4_um_global_power_events = {
     { .type = perfctr_um_type_bitmask,
       .default_value = 0x01,
@@ -259,6 +222,16 @@ static const struct perfctr_unit_mask_3 p4_um_uop_queue_writes = {
       { 0x04, "FROM_ROM:uops written from microcode ROM" } }
 };
 
+static const struct perfctr_unit_mask_4 p4_um_branch_type = {
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x1E,
+      .nvalues = 4 },
+    { { 0x02, "CONDITIONAL:Conditional jumps" },
+      { 0x04, "CALL:Call branches" }, /* XXX: diff MISPRED/non-MISPRED events? */
+      { 0x08, "RETURN:Return branches" },
+      { 0x10, "INDIRECT:Returns, indirect calls, or indirect jumps" } }
+};
+
 static const struct perfctr_unit_mask_1 p4_um_resource_stall = {
     { .type = perfctr_um_type_bitmask,
       .default_value = 0x20,
@@ -272,6 +245,8 @@ static const struct perfctr_unit_mask_3 p4_um_WC_Buffer = {
       .nvalues = 3 },
     { { 0x01, "WCB_EVICTS:all causes" },
       { 0x02, "WCB_FULL_EVICT:no WC buffer is available" },
+      /* XXX: 245472-011 no longer lists bit 2, but that looks like
+	 a table formatting error. Keeping it for now. */
       { 0x04, "WCB_HITM_EVICT:store encountered a Hit Modified condition" } }
 };
 
@@ -315,21 +290,45 @@ static const struct perfctr_unit_mask_2 p4_um_uop_type = {
       { 0x04, "TAGSTORES:The uop is a store operation" } }
 };
 
-static const struct perfctr_unit_mask_4 p4_um_branch_type = {
+static const struct perfctr_unit_mask_4 p4_um_branch_retired = {
     { .type = perfctr_um_type_bitmask,
-      .default_value = 0x1E,
+      .default_value = 0x0C,	/* taken branches */
       .nvalues = 4 },
-    { { 0x02, "CONDITIONAL:Conditional jumps" },
-      { 0x04, "CALL:Call branches" }, /* XXX: diff MISPRED/non-MISPRED events? */
-      { 0x08, "RETURN:Return branches" },
-      { 0x10, "INDIRECT:Returns, indirect calls, or indirect jumps" } }
+    { { 0x01, "MMNP:Branch Not-taken Predicted" },
+      { 0x02, "MMNM:Branch Not-taken Mispredicted" },
+      { 0x04, "MMTP:Branch Taken Predicted" },
+      { 0x08, "MMTM:Branch Taken Mispredicted" } }
+};
+
+static const struct perfctr_unit_mask_1 p4_um_mispred_branch_retired = {
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x01,
+      .nvalues = 1 },
+    { { 0x01, "NBOGUS:The retired branch is not bogus" } }
+};
+
+static const struct perfctr_unit_mask_5 p4_um_x87_assist = {
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x1F,
+      .nvalues = 5 },
+    { { 0x01, "FPSU:FP stack underflow" },
+      { 0x02, "FPSO:FP stack overflow" },
+      { 0x04, "POAO:x87 output overflow" },
+      { 0x08, "POAU:x87 output underflow" },
+      { 0x10, "PREA:x87 input assist" } }
+};
+
+static const struct perfctr_unit_mask_3 p4_um_machine_clear = {
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x01,
+      .nvalues = 3 },
+    { { 0x01, "CLEAR:Count a portion of the cycles when the machine is cleared" },
+      { 0x04, "MOCLEAR:Count clears due to memory ordering issues" },
+      { 0x08, "SMCLEAR:Count clears due to self-modifying code issues" } }
 };
 
 static const struct perfctr_event p4_events[] = {
-    { 0x06, CRU_ESCR_2_3, UM(p4_um_branch_retired), "branch_retired",
-      "retired branches" },
-    { 0x03, CRU_ESCR_0_1, UM(p4_um_mispred_branch_retired), "mispred_branch_retired",
-      "retired mispredicted branches" },
+    /* Non-Retirement Events: */
     { 0x01, TC_ESCR_0_1, UM(p4_um_TC_deliver_mode), "TC_deliver_mode",
       "duration of the operating modes of the trace cache and decode engine" },
     { 0x03, BPU_ESCR_0_1, UM(p4_um_BPU_fetch_request), "BPU_fetch_request",
@@ -341,16 +340,19 @@ static const struct perfctr_event p4_events[] = {
     { 0x08, SAAT_ESCR_0_1, UM(p4_um_memory_complete), "memory_complete",
       "completed load split, store split, uncacheable split, uncacheable load" },
     { 0x04, SAAT_ESCR_0_1, UM(p4_um_load_port_replay), "load_port_replay",
-      "replayed events at the load port" }, /* XXX: only ESCR1 supports at-retirement */
+      /* XXX: only ESCR1 supports at-retirement */
+      "replayed events at the load port" },
     { 0x05, SAAT_ESCR_0_1, UM(p4_um_store_port_replay), "store_port_replay",
-      "replayed events at the store port" }, /* XXX: only ESCR1 supports at-retirement */
+      /* XXX: only ESCR1 supports at-retirement */
+      "replayed events at the store port" },
     { 0x03, MOB_ESCR_0_1, UM(p4_um_MOB_load_replay), "MOB_load_replay",
       "replayed loads at the memory order buffer" },
     { 0x01, PMH_ESCR_0_1, UM(p4_um_page_walk_type), "page_walk_type",
       "page walks by the page miss handler" },
     { 0x0C, BSU_ESCR_0_1, UM(p4_um_BSQ_cache_reference), "BSQ_cache_reference",
       "cache references seen by the bus unit" },
-    { 0x03, FSB_ESCR_0, UM(p4_um_IOQ), "IOQ_allocation",
+    { 0x03, FSB_ESCR_0_1, UM(p4_um_IOQ), "IOQ_allocation",
+      /* XXX: ESCR1 unavailable if CPUID < 0xF27 */
       "bus transactions" },
     { 0x1A, FSB_ESCR_1, UM(p4_um_IOQ), "IOQ_active_entries",
       "number of active IOQ entries" },
@@ -360,8 +362,6 @@ static const struct perfctr_event p4_events[] = {
       "allocations in the bus sequence unit" },
     { 0x06, BSU_ESCR_1, UM(p4_um_BSQ), "bsq_active_entries",
       "number of active BSQ entries" },
-    { 0x03, CRU_ESCR_2_3, UM(p4_um_x87_assist), "x87_assist",
-      "retired x87 instructions that required special handling" },
     { 0x34, FIRM_ESCR_0_1, UM(p4_um_firm_uop), "SSE_input_assist",
       "assists requested for SSE and SSE2 input operands" },
     { 0x08, FIRM_ESCR_0_1, UM(p4_um_firm_uop), "packed_SP_uop",
@@ -380,19 +380,33 @@ static const struct perfctr_event p4_events[] = {
       "x87 floating-point uops" },
     { 0x2E, FIRM_ESCR_0_1, UM(p4_um_x87_SIMD_moves_uop), "x87_SIMD_moves_uop",
       "x87 FPU, MMX, SSE, or SSE2 load, store, and move uops" },
-    { 0x02, CRU_ESCR_2_3, UM(p4_um_machine_clear), "machine_clear",
-      "cycles or occurrences when the entire pipeline is cleared" },
     { 0x05, FSB_ESCR_0_1, UM(p4_um_global_power_events), "global_power_events",
       "time during which the processor is not stopped" },
     { 0x05, MS_ESCR_0_1, UM(p4_um_tc_ms_xfer), "tc_ms_xfer",
       "number of times uop delivery changed from TC to MS ROM" },
     { 0x09, MS_ESCR_0_1, UM(p4_um_uop_queue_writes), "uop_queue_writes",
       "number of valid uops written to the uop queue" },
+    { 0x05, TBPU_ESCR_0_1, UM(p4_um_branch_type), "retired_mispred_branch_type",
+      "retired mispredicted branches by type" },
+    { 0x04, TBPU_ESCR_0_1, UM(p4_um_branch_type), "retired_branch_type",
+      "retired branches by type" },
     { 0x01, ALF_ESCR_0_1, UM(p4_um_resource_stall), "resource_stall",
       /* XXX: may not be supported in all P4 models */
       "stalls in the Allocator" },
     { 0x05, DAC_ESCR_0_1, UM(p4_um_WC_Buffer), "WC_Buffer",
       "write combining buffer operations" },
+    { 0x16, FSB_ESCR_0_1, NULL, "b2b_cycles",
+      /* XXX: may not be supported in all P4 models */
+      "back-to-back bus cycles" },
+    { 0x04, FSB_ESCR_0_1, NULL, "bnr",
+      /* XXX: may not be supported in all P4 models */
+      "bus not ready conditions" },
+    { 0x06, FSB_ESCR_0_1, NULL, "snoop",
+      /* XXX: may not be supported in all P4 models */
+      "snoop hit modified bus traffic" },
+    { 0x04, FSB_ESCR_0_1, NULL, "response",
+      /* XXX: may not be supported in all P4 models */
+      "different types of responses" },
     { 0x08, CRU_ESCR_2_3, UM(p4_um_nbogus_bogus), "front_end_event",
       /* XXX: another ESCR must count uop_type */
       /* XXX: can support PEBS */
@@ -411,10 +425,14 @@ static const struct perfctr_event p4_events[] = {
       "retired uops" },
     { 0x02, RAT_ESCR_0_1, UM(p4_um_uop_type), "uop_type",
       "tag uops for the front-end tagging mechanism" },
-    { 0x05, TBPU_ESCR_0_1, UM(p4_um_branch_type), "retired_mispred_branch_type",
-      "retired mispredicted branches by type" },
-    { 0x04, TBPU_ESCR_0_1, UM(p4_um_branch_type), "retired_branch_type",
-      "retired branches by type" },
+    { 0x06, CRU_ESCR_2_3, UM(p4_um_branch_retired), "branch_retired",
+      "retired branches" },
+    { 0x03, CRU_ESCR_0_1, UM(p4_um_mispred_branch_retired), "mispred_branch_retired",
+      "retired mispredicted branches" },
+    { 0x03, CRU_ESCR_2_3, UM(p4_um_x87_assist), "x87_assist",
+      "retired x87 instructions that required special handling" },
+    { 0x02, CRU_ESCR_2_3, UM(p4_um_machine_clear), "machine_clear",
+      "cycles or occurrences when the entire pipeline is cleared" },
 };
 
 const struct perfctr_event_set perfctr_p4_event_set = {
